/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by VIA Networking Technologies, Inc.
 * 4. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY VIA NETWORKING TECHNOLOGIES, INC. AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * VIA NETWORKING TECHNOLOGIES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * File: if_getb.c
 *
 * Purpose: FreeBSD driver main source file.
 *
 * Author: Ryan Fu
 *
 * Date: Jul 22, 2003
 *
 * Functions:
 *      getb_intr
 *      getb_media_status
 *      getb_media_change
 *      getb_start
 *      getb_ioctl
 *      getb_watchdog
 *      getb_init
 *      getb_stop
 *      getb_init_rings
 *      getb_init_rd_ring
 *      getb_init_td_ring
 *      getb_free_td_ring
 *      getb_free_rd_ring
 *      getb_free_rings
 *      getb_init_registers
 *      getb_setup_interface
 *      getb_set_multi
 *      getb_rx_srv
 *      getb_tx_srv
 *      getb_error
 *      getb_receive_frame
 *      getb_rx_csum
 *      getb_get_opt_media_mode
 *      mac_check_media_mode
 *      getb_set_media_mode
 *      getb_print_link_status
 *      mii_check_media_mode
 *      enable_flow_control_ability
 *      mii_init
 *      getb_mii_read
 *      getb_mii_write
 *      getb_soft_reset
 *      getb_init_cam_filter
 *      getb_mac_shutdown
 *      EnableMiiAutoPoll
 *
 * Revision History:
 *      07-22-2003 Ryan Fu: Add new copyright header
 *
 *
 */

#include "if_getb.h"



/*---------------------------- Definitions -----------------------------*/

static char getb_version[]=GETB_VERSION;

PGETB_INFO  pgetb_infos=NULL;

static CHIP_INFO chip_info_table[]= {
    { CHIP_TYPE_VT6110, GETB_ADAPTER_NAM,
        256, 1, 0x00FFFFFFUL},
    {0,NULL}
};

static GETB_VENDOR_INFO getb_devs[] = {
    { VIA_VENDOR_ID, DEVICE_ID_VT6110, PCI_ANY_ID, PCI_ANY_ID,
        &chip_info_table[0]},
    { 0, 0, 0, 0, NULL }
};


#if __FreeBSD_version >= 400000

static int  getb_probe              __P((device_t));
static int  getb_attach             __P((device_t));
static int  getb_detach             __P((device_t));
static int  getb_shutdown           __P((device_t));

static BOOL getb_get_chip           __P((device_t dev, PCHIP_INFO* ppChip));
static BOOL getb_init_info          __P((device_t dev, PGETB_INFO* ppInfo, PCHIP_INFO pChip_info));
static int  getb_get_pci_info       __P((PGETB_INFO pInfo, device_t dev));
static void getb_free_pci_info      __P((PGETB_INFO pInfo));

#elif __FreeBSD_version < 400000

static unsigned long  getb_count = 0;

static const char    *getb_probe    __P((pcici_t, pcidi_t));
static void           getb_attach   __P((pcici_t, int));
static void           getb_shutdown __P((int, void *));

static BOOL           getb_get_chip __P((pcidi_t devid, PCHIP_INFO* ppChip));
static BOOL           getb_init_info __P((int unit, PGETB_INFO* ppInfo, PCHIP_INFO pChip_info));

#endif /*__FreeBSD_version*/

static void getb_intr               __P((void *));
static void getb_media_status       __P((struct ifnet *ifp, struct ifmediareq *ifmr));
static int  getb_media_change       __P((struct ifnet *ifp));
static void getb_start              __P((struct ifnet *));
static int  getb_ioctl              __P((struct ifnet *, u_long, caddr_t));
static void getb_watchdog           __P((struct ifnet *));
static void getb_init               __P((void *));
static void getb_stop               __P((void *));

static BOOL getb_init_rings         __P((PGETB_INFO pInfo));
static BOOL getb_init_rd_ring       __P((PGETB_INFO pInfo));
static BOOL getb_init_td_ring       __P((PGETB_INFO pInfo));
static void getb_free_td_ring       __P((PGETB_INFO pInfo));
static void getb_free_rd_ring       __P((PGETB_INFO pInfo));
static void getb_free_rings         __P((PGETB_INFO pInfo));
static void getb_init_registers     __P((PGETB_INFO pInfo, GETB_INIT_TYPE InitType));
static void getb_setup_interface    __P((PGETB_INFO pInfo));
static void getb_set_multi          __P((PGETB_INFO pInfo));
static int  getb_rx_srv             __P((PGETB_INFO pInfo));
static int  getb_tx_srv             __P((PGETB_INFO pInfo));
static void getb_error              __P((PGETB_INFO pInfo, u_int32_t status));
static BOOL getb_receive_frame      __P((PGETB_INFO pInfo, int idx));
static BOOL getb_alloc_rx_buf       __P((PGETB_INFO pInfo, int idx, struct mbuf* omp));

#if __FreeBSD_version >= 400000
static __inline void getb_rx_csum   __P((uint32_t, struct mbuf* mp));
#endif


static struct mbuf* defrag_mbuf     __P((struct mbuf* mp));

#if __FreeBSD_version >= 400000

#if __FreeBSD_version >= 502000
MODULE_DEPEND(getb, pci, 1, 1, 1);
MODULE_DEPEND(getb, ether, 1, 1, 1);
#endif

static device_method_t getb_methods[] = {
   /* Device interface */
   DEVMETHOD(device_probe, getb_probe),
   DEVMETHOD(device_attach, getb_attach),
   DEVMETHOD(device_detach, getb_detach),
   DEVMETHOD(device_shutdown, getb_shutdown),
   {0, 0}
};

static driver_t getb_driver = {
   "getb", getb_methods, sizeof(GETB_INFO)
};

static devclass_t getb_devclass;

DRIVER_MODULE(getb, pci, getb_driver, getb_devclass, 0, 0);

#elif __FreeBSD_version < 400000

static struct pci_device getb_device = {
    "getb",
    getb_probe,
    getb_attach,
    &getb_count,
    NULL
};

DATA_SET(pcidevice_set, getb_device);

#endif /*__FreeBSD_version*/




/*---------------------------- Functions ----------------------------------*/


#if __FreeBSD_version >= 400000

static BOOL getb_get_chip(device_t dev, PCHIP_INFO* ppChip)
{
    U16                 wDevId, wVenId, wSubVenId, wSubDevId;
    PGETB_VENDOR_INFO   ptr;


    wVenId=pci_get_vendor(dev);

    if (wVenId!=(U16)VIA_VENDOR_ID)
        return FALSE;

    wDevId = pci_get_device(dev);
    wSubVenId = pci_get_subvendor(dev);
    wSubDevId = pci_get_subdevice(dev);

    for (ptr=&getb_devs[0]; ptr->wVendorId!=0; ptr++) {
        if ((wVenId == ptr->wVendorId) &&
            (wDevId == ptr->wDeviceId) &&
            ((wSubVenId == ptr->wSubVenId) || (ptr->wSubVenId == PCI_ANY_ID)) &&
            ((wSubDevId == ptr->wSubDevId) || (ptr->wSubDevId == PCI_ANY_ID))) {

            if (ppChip!=NULL)
                *ppChip=ptr->pChip;

            return TRUE;
         }
    }

    return FALSE;
}

static BOOL getb_init_info(device_t dev, PGETB_INFO* ppInfo, PCHIP_INFO pChip_info)
{
    PGETB_INFO  p;

    if (pgetb_infos==NULL) {
        pgetb_infos = *ppInfo;
    }
    else {
        for (p=pgetb_infos; p->next!=NULL; p=p->next)
            do {} while (0);

        p->next = *ppInfo;
        (*ppInfo)->prev=p;
    }

    (*ppInfo)->osdep.dev=dev;
    (*ppInfo)->unit=device_get_unit(dev);
    (*ppInfo)->chip_id=pChip_info->chip_id;
    (*ppInfo)->hw.io_size=pChip_info->io_size;
    (*ppInfo)->hw.nTxQueues=pChip_info->nTxQueue;
    (*ppInfo)->multicast_limit=MCAM_SIZE;
    (*ppInfo)->rx_buf_sz=RX_BUF_SIZE_DEFAULT;
    (*ppInfo)->hw.back = &(*ppInfo)->osdep;
    /*Init Options*/
    (*ppInfo)->hw.sOpts.nTxDescs=TX_DESC_NUM_DEF;
    (*ppInfo)->hw.sOpts.nRxDescs=(RX_DESC_NUM_DEF+3) & ~3;
    (*ppInfo)->hw.sOpts.spd_dpx=SPEED_DUPLEX_DEF;
    (*ppInfo)->hw.sOpts.DMA_length=DMA_LENGTH_DEF;
    (*ppInfo)->hw.sOpts.rx_thresh=RX_THRESH_DEF;
    (*ppInfo)->hw.sOpts.flow_cntl=FLOW_CNTL_DEF;
    (*ppInfo)->hw.sOpts.flags=0;

#if VAL_PKTLEN_DEF == 1
    (*ppInfo)->hw.sOpts.flags|=GETB_FLAGS_VAL_PKT_LEN;
#endif

#if HWCSUM_OFFLOAD == 1
    (*ppInfo)->hw.sOpts.flags|=GETB_FLAGS_TX_CSUM|GETB_FLAGS_RX_CSUM;
#endif

#if __FreeBSD_version >= 500000 && VLAN_TAGGING == 1
    (*ppInfo)->hw.sOpts.flags|=GETB_FLAGS_VLANTAG;
#endif

    (*ppInfo)->hw.flags=(*ppInfo)->hw.sOpts.flags;

    callout_handle_init(&((*ppInfo)->timer_handle));

    return TRUE;
}

static int  getb_get_pci_info(PGETB_INFO pInfo, device_t dev)
{
    U16 pci_cmd;
    int res_id=GETB_MEMIO_RES_ID;

    pInfo->hw.byRevId = pci_read_config(dev, GETB_PCI_REVID, 1);
    pInfo->hw.SubVendorID = pci_get_subvendor(dev);
    pInfo->hw.SubSystemID = pci_get_subdevice(dev);


#if __FreeBSD_version >= 500000
    pci_cmd = pci_read_config(dev, GETB_PCI_COMMAND, 2);
    pci_enable_busmaster(dev);
#else
    pci_cmd = pci_read_config(dev, PCIR_COMMAND, 2);

    if (!((pci_cmd & PCIM_CMD_BUSMASTEREN) && (pci_cmd & PCIM_CMD_MEMEN))) {
        GETB_PRT0(pInfo, "Memory Access and/or Bus Master bits were not set!\n");
        pci_cmd |= (PCIM_CMD_BUSMASTEREN | PCIM_CMD_MEMEN);
        pci_write_config(dev, PCIR_COMMAND, pci_cmd, 2);
    }
#endif

    pInfo->mem_res = bus_alloc_resource(dev, SYS_RES_MEMORY, &res_id, 0, ~0, 1, RF_ACTIVE);

    if (pInfo->mem_res==NULL) {
        GETB_PRT0(pInfo, "Unable to allocate bus resource: memory\n");
        goto fail;
    }

    pInfo->osdep.mem_bus_space_tag=rman_get_bustag(pInfo->mem_res);
    pInfo->osdep.mem_bus_space_handle=rman_get_bushandle(pInfo->mem_res);

    res_id=0;
    pInfo->int_res = bus_alloc_resource(dev, SYS_RES_IRQ, &res_id, 0, ~0, 1, RF_SHAREABLE | RF_ACTIVE);

    if (pInfo->int_res==NULL) {
        GETB_PRT0(pInfo, "Unable to allocate bus resource: interrupt\n");
        goto fail;
    }

    if (bus_setup_intr(dev, pInfo->int_res, INTR_TYPE_NET,
                       (void (*)(void *))getb_intr, pInfo,
                       &pInfo->int_hdl_tag)) {

        GETB_PRT0(pInfo, "Error registering interrupt handler!\n");
        goto fail;
    }

    return 0;

fail:
    getb_free_pci_info(pInfo);
    return(ENXIO);

}

static void getb_free_pci_info(PGETB_INFO pInfo)
{
    if (pInfo->int_hdl_tag) {
        bus_teardown_intr(pInfo->hw.back->dev, pInfo->int_res, pInfo->int_hdl_tag);
    }

    if (pInfo->int_res) {
        bus_release_resource(pInfo->hw.back->dev, SYS_RES_IRQ, 0, pInfo->int_res);
    }

    if (pInfo->mem_res) {
        bus_release_resource(pInfo->hw.back->dev, SYS_RES_MEMORY, GETB_MEMIO_RES_ID, pInfo->mem_res);
    }
}

static int getb_probe(device_t dev)
{
    char                buf[256];
    PCHIP_INFO          pChip;

    if (getb_get_chip(dev, &pChip)) {
        sprintf(buf, "%s, Version - %s", pChip->name, getb_version);
        device_set_desc_copy(dev, buf);
        return(0);
    }

    return ENXIO;
}

#elif __FreeBSD_version < 400000

static BOOL getb_get_chip(pcidi_t devid, PCHIP_INFO* ppChip)
{
    U16                 wDevId, wVenId;
    PGETB_VENDOR_INFO   ptr;

    wVenId=(U16)(devid & 0xFFFF);

    if (wVenId!=(U16)VIA_VENDOR_ID)
        return FALSE;

    wDevId = (U16)((devid >> 16) & 0xFFFF);

    for (ptr=&getb_devs[0]; ptr->wVendorId!=0; ptr++) {
        if ((wVenId == ptr->wVendorId) && (wDevId == ptr->wDeviceId)) {
            if (ppChip!=NULL)
                *ppChip=ptr->pChip;
            return TRUE;
         }
    }

    return FALSE;
}

static BOOL getb_init_info(int unit, PGETB_INFO* ppInfo, PCHIP_INFO pChip_info)
{
    PGETB_INFO  p;

    if (pgetb_infos==NULL) {
        pgetb_infos = *ppInfo;
    }
    else {
        for (p=pgetb_infos; p->next!=NULL; p=p->next)
            do {} while (0);

        p->next = *ppInfo;
        (*ppInfo)->prev=p;
    }

    /*(*ppInfo)->dev=dev;*/
    (*ppInfo)->unit=unit;
    (*ppInfo)->osdep.unit=unit;
    (*ppInfo)->chip_id=pChip_info->chip_id;
    (*ppInfo)->hw.io_size=pChip_info->io_size;
    (*ppInfo)->hw.nTxQueues=pChip_info->nTxQueue;
    (*ppInfo)->multicast_limit=MCAM_SIZE;
    (*ppInfo)->rx_buf_sz=RX_BUF_SIZE_DEFAULT;
    (*ppInfo)->hw.back = &(*ppInfo)->osdep;

    /*Init Options*/
    (*ppInfo)->hw.sOpts.nTxDescs=TX_DESC_NUM_DEF;
    (*ppInfo)->hw.sOpts.nRxDescs=(RX_DESC_NUM_DEF+3) & ~3;
    (*ppInfo)->hw.sOpts.spd_dpx=SPEED_DUPLEX_DEF;
    (*ppInfo)->hw.sOpts.DMA_length=DMA_LENGTH_DEF;
    (*ppInfo)->hw.sOpts.rx_thresh=RX_THRESH_DEF;
    (*ppInfo)->hw.sOpts.flow_cntl=FLOW_CNTL_DEF;
    (*ppInfo)->hw.sOpts.flags=0;

#if VAL_PKTLEN_DEF == 1
    (*ppInfo)->hw.sOpts.flags|=GETB_FLAGS_VAL_PKT_LEN;
#endif

    (*ppInfo)->hw.flags=(*ppInfo)->sOpts.flags;

    /*callout_handle_init(&((*ppInfo)->timer_handle));*/

    return TRUE;
}

static const char * getb_probe(pcici_t config_id, pcidi_t device_id)
{
    PCHIP_INFO          pChip;

    if (getb_get_chip(device_id, &pChip)) {
        printf("%s, Version - %s\n", pChip->name, getb_version);
        return(pChip->name);
    }

    return(NULL);
}

#endif /*__FreeBSD_version*/



#if __FreeBSD_version >= 400000

static int getb_attach(device_t dev)
{
    int         i, error=0;
    PGETB_INFO  pInfo=device_get_softc(dev);
    PCHIP_INFO  pChip;
    int         status;

#if __FreeBSD_version >= 500000
    mtx_init(&pInfo->get_mtx, device_get_nameunit(dev), MTX_NETWORK_LOCK, MTX_DEF | MTX_RECURSE);
#else
    pInfo->get_splcount=0;
#endif

    GET_LOCK(pInfo);

    if (pInfo==NULL) {
        printf("getb: Driver infomation structure allocation failed\n");
        error=ENOMEM;
        goto fail;
    }

    /* search for chip information using PCI Device ID, Vendor ID*/
    if (!getb_get_chip(dev, &pChip)) {
        error=ENXIO;
        goto fail;
    }

    /* init GETB_INFO device private data structure */
    getb_init_info(dev, &pInfo, pChip);

    /* setup PCI resources(mem, io, irq) */
    if ((status=getb_get_pci_info(pInfo, dev))) {
        error=status;
        goto fail;
    }

    /* allocate RD/TD rings */
    if (!getb_init_rings(pInfo)) {
        getb_free_pci_info(pInfo);
        error=ENOMEM;
        goto fail;
    }

    /* init RD ring & allocate rx buffer*/
    if (!getb_init_rd_ring(pInfo)) {
        getb_free_pci_info(pInfo);
        getb_free_rings(pInfo);
        error=ENOMEM;
        goto fail;
    }

    /* init TD ring */
    if (!getb_init_td_ring(pInfo)) {
        getb_free_rd_ring(pInfo);
        getb_free_rings(pInfo);
        getb_free_pci_info(pInfo);
        error=ENOMEM;
        goto fail;
    }

    /* Reset the adapter. */
    /*getb_soft_reset(pInfo);*/
    velocity_soft_reset(&pInfo->hw);
    
    mac_eeprom_reload(&pInfo->hw);

    /* EEPROM reloaded will cause bit 0 in MAC_REG_CFGA turned on.
     * it makes MAC receive magic packet automatically. So, driver turn it off.
     */
    BYTE_REG_BITS_OFF(&pInfo->hw, CFGA_PACPI, MAC_REG_CFGA);


    for (i = 0; i < 6; i++)
        pInfo->arpcom.ac_enaddr[i] = CSR_READ_1(&pInfo->hw, MAC_REG_PAR+i);

    /* setup ifnet interface */
    getb_setup_interface(pInfo);

    pInfo->hw.dwPHYId=MII_GET_PHY_ID(&pInfo->hw);

    /* Print out mii status */
    mac_disable_int(&pInfo->hw);
    EnableMiiAutoPoll(&pInfo->hw);
    pInfo->hw.mii_status=check_connectiontype(&pInfo->hw);

    if (!(pInfo->hw.mii_status & VELOCITY_LINK_FAIL))
        velocity_print_link_status(&pInfo->hw);

    /* Clear Hardware MIB counter */
    mac_hw_mibs_init(&pInfo->hw);

    GET_UNLOCK(pInfo);
    return 0;

fail:
    GET_UNLOCK(pInfo);
#if __FreeBSD_version >= 500000
    mtx_destroy(&pInfo->get_mtx);
#endif
    return(error);
}


static int getb_detach(device_t dev)
{
    PGETB_INFO     pInfo=device_get_softc(dev);
    struct ifnet   *ifp = &pInfo->arpcom.ac_if;

    GET_LOCK(pInfo);

    ifp->if_flags &= ~(IFF_RUNNING | IFF_OACTIVE | IFF_UP);
    ifp->if_timer = 0;

    getb_stop(pInfo);

#if __FreeBSD_version >= 400000 && __FreeBSD_version < 410000
    if_detach(ifp);
#elif __FreeBSD_version >= 410000 && __FreeBSD_version < 500000
    ether_ifdetach(ifp, ETHER_BPF_SUPPORTED);
#elif __FreeBSD_version >= 500000
    ether_ifdetach(ifp);
#endif /*__FreeBSD_version*/

    getb_free_td_ring(pInfo);
    getb_free_rd_ring(pInfo);
    getb_free_rings(pInfo);
    getb_free_pci_info(pInfo);

    GET_UNLOCK(pInfo);

#if __FreeBSD_version >= 500000
    mtx_destroy(&pInfo->get_mtx);
#endif

    return 0;
}

#elif __FreeBSD_version < 400000

static void getb_attach(pcici_t config_id, int unit)
{
    int             i;
    vm_offset_t     pbase, vbase;
    u_int32_t       pci_cmd, id;

    PGETB_INFO  pInfo;
    PCHIP_INFO  pChip = &chip_info_table[0];


    id = pci_conf_read(config_id, GETB_PCI_VENDOR_ID);

    if (PCI_VENDORID(id) != VIA_VENDOR_ID) {
        printf("getb%d: wrong vendor ID !\n", unit);
        return;
    }

    if (PCI_CHIPID(id) != DEVICE_ID_VT6110) {
        printf("getb%d: wrong device ID !\n", unit);
        return;
    }

    pInfo = (PGETB_INFO)malloc(sizeof(GETB_INFO), M_DEVBUF, M_NOWAIT);

    if (pInfo==NULL) {
        printf("getb%d: Driver infomation structure allocation failed\n", unit);
        return;
    }

    bzero(pInfo, sizeof(GETB_INFO));

    GET_LOCK(pInfo);

    /* init GETB_INFO device private data structure */
    getb_init_info(unit, &pInfo, pChip);

    /* setup PCI resources(mem, io, irq) */

    pInfo->hw.byRevId = pci_conf_read(config_id, GETB_PCI_REVID) & 0x000000FF;

    pci_cmd = pci_conf_read(config_id, PCI_COMMAND_STATUS_REG);

    if (!((pci_cmd & PCIM_CMD_BUSMASTEREN) && (pci_cmd & PCIM_CMD_MEMEN))) {
        GETB_PRT0(pInfo, "Memory Access and/or Bus Master bits were not set!\n");
        pci_cmd |= (PCIM_CMD_BUSMASTEREN | PCIM_CMD_MEMEN);
        pci_conf_write(config_id, PCI_COMMAND_STATUS_REG, pci_cmd);
    }

    if (!pci_map_mem(config_id, GETB_PCI_LOMEM, &vbase, &pbase)) {
        printf ("getb%d: couldn't map memory\n", unit);
        goto fail;
    }

    pInfo->osdep.mem_bus_space_tag=I386_BUS_SPACE_MEM;
    pInfo->osdep.mem_bus_space_handle=vbase;

    /* Allocate interrupt */
    if (!pci_map_int(config_id, getb_intr, pInfo, &net_imask)) {
        printf("getb%d: couldn't map interrupt\n", unit);
        goto fail;
    }

    /* allocate RD/TD rings */
    if (!getb_init_rings(pInfo)) {
        printf("getb%d: couldn't allocate RD/TD rings\n", unit);
        goto fail;
    }

    /* init RD ring & allocate rx buffer*/
    if (!getb_init_rd_ring(pInfo)) {
        printf("getb%d: couldn't init RD ring & allocate rx buffer\n", unit);
        getb_free_rings(pInfo);
        goto fail;
    }

    /* init TD ring */
    if (!getb_init_td_ring(pInfo)) {
        printf("getb%d: couldn't init TD ring\n", unit);
        getb_free_rd_ring(pInfo);
        getb_free_rings(pInfo);
        goto fail;
    }

    /* Reset the adapter. */
    velocity_soft_reset(&pInfo->hw);
    mac_eeprom_reload(&pInfo->hw);

    /* EEPROM reloaded will cause bit 0 in MAC_REG_CFGA turned on.
     * it makes MAC receive magic packet automatically. So, driver turn it off.
     */
    BYTE_REG_BITS_OFF(&pInfo->hw, CFGA_PACPI, MAC_REG_CFGA);

    for (i = 0; i < 6; i++)
        pInfo->arpcom.ac_enaddr[i] = CSR_READ_1(&pInfo->hw, MAC_REG_PAR+i);

    /* setup ifnet interface */
    getb_setup_interface(pInfo);

    pInfo->hw.dwPHYId=MII_GET_PHY_ID(pInfo);

    /* Print out mii status */
    mac_disable_int(&pInfo->hw);
    EnableMiiAutoPoll(&pInfo->hw);
    pInfo->hw.mii_status=check_connectiontype(&pInfo->hw);

    if (!(pInfo->hw.mii_status & VELOCITY_LINK_FAIL))
        velocity_print_link_status(&pInfo->hw);

    /* Clear Hardware MIB counter */
    mac_hw_mibs_init(pInfo);

    GET_UNLOCK(pInfo);
    return;

fail:
    GET_UNLOCK(pInfo);
    free(pInfo, M_DEVBUF);
    return;
}

#endif /*__FreeBSD_version*/


#if __FreeBSD_version >= 400000

static int  getb_shutdown(device_t dev)
{
    PGETB_INFO  pInfo=device_get_softc(dev);

    getb_stop(pInfo);

    return 0;
}

#elif __FreeBSD_version < 400000

static void getb_shutdown(int howto, void * arg)
{
    PGETB_INFO  pInfo=(PGETB_INFO)arg;

    getb_stop(pInfo);

    return;
}

#endif /*__FreeBSD_version*/


static void getb_intr (void * arg)
{
    PGETB_INFO      pInfo;
    struct ifnet*   ifp;
    u_int32_t       isr_status;

    pInfo = arg;

    GET_LOCK(pInfo);
    ifp = &pInfo->arpcom.ac_if;


    if (!(ifp->if_flags & IFF_UP)) {
        GET_UNLOCK(pInfo);
        return;
    }

    /* Disable interrupts */
    mac_disable_int(&pInfo->hw);

    for (;;) {

        isr_status=mac_read_isr(&pInfo->hw);



        /* If the card has gone away the read returns 0xffff. */
        if (isr_status == 0xFFFFFFFF)
            break;

        if (isr_status)
            mac_write_isr(&pInfo->hw, isr_status);

        if ((isr_status & pInfo->hw.IntMask) == 0)
            break;

        /*No need to check ISR status for Rx, since we'll check RD status in Rx service routine.*/
        /*if (isr_status & (ISR_PRXI|ISR_PPRXI))*/
            getb_rx_srv(pInfo);

        if (isr_status & (ISR_PTXI|ISR_PPTXI))
            getb_tx_srv(pInfo);

        if (isr_status & ~(ISR_PRXI|ISR_PPRXI|ISR_PTXI|ISR_PPTXI)) {
            getb_error(pInfo, isr_status);
        }
    }

    /* Re-enable interrupts */
    mac_enable_int(&pInfo->hw);

    GET_UNLOCK(pInfo);

    if (ifp->if_flags & IFF_RUNNING && ifp->if_snd.ifq_head != NULL)
        getb_start(ifp);

}


static int getb_rx_srv (PGETB_INFO pInfo)
{
    int                     works=0;
    PRX_DESC                pRD;
    struct  ifnet*          ifp = &(pInfo->arpcom.ac_if);
    int                     iCurrRDIdx=pInfo->hw.iCurrRDIdx;

    while (TRUE) {
        pRD = &(pInfo->hw.aRDRing[iCurrRDIdx]);

        if (pRD->rdesc0 & RDESC0_OWN)
            break;

        if (!getb_receive_frame(pInfo, iCurrRDIdx))
            ifp->if_ierrors++;

        pRD->rdesc3 |= cpu_to_le32(RDESC3_INTCTL);

        /* Append 4 RDs each time */
        if ((iCurrRDIdx % 4) == 3) {
            int i, iPrevRDIdx = iCurrRDIdx;
            for (i=0; i<4; i++) {
                pRD = &(pInfo->hw.aRDRing[iPrevRDIdx]);
                pRD->rdesc0 |= RDESC0_OWN;
                if (iPrevRDIdx <= 0)
                    iPrevRDIdx = pInfo->hw.sOpts.nRxDescs - 1;
                else
                    --iPrevRDIdx;
            }
            CSR_WRITE_2(&pInfo->hw, 4, MAC_REG_RBRDU);
        }

        iCurrRDIdx = (iCurrRDIdx+1) % pInfo->hw.sOpts.nRxDescs;

        if (++works == pInfo->hw.sOpts.nRxDescs)
            break;

    } /* while */

    pInfo->hw.iCurrRDIdx=iCurrRDIdx;

    return works;
}


#if __FreeBSD_version >= 400000
static __inline void getb_rx_csum(uint32_t ctl, struct mbuf* mp)
{
    uint8_t rxctl;
    mp->m_pkthdr.csum_flags=0;

    rxctl = (uint8_t)ctl;

    /* Check IP header checksum */
    if (rxctl & CSM_IPKT)
        mp->m_pkthdr.csum_flags |= CSUM_IP_CHECKED;
    if (rxctl & CSM_IPOK)
        mp->m_pkthdr.csum_flags |= CSUM_IP_VALID;

    /* Check TCP/UDP checksum */
    if (rxctl & (CSM_TCPKT|CSM_UDPKT) &&
        rxctl & CSM_TUPOK) {
        mp->m_pkthdr.csum_flags |= CSUM_DATA_VALID|CSUM_PSEUDO_HDR;
        mp->m_pkthdr.csum_data = 0xffff;
    }
}
#endif

static BOOL getb_receive_frame(PGETB_INFO pInfo, int idx)
{
    PGETB_RD_INFO           pRDInfo = &(pInfo->aRDInfo[idx]);
    PRX_DESC                pRD = &(pInfo->hw.aRDRing[idx]);
    struct  ifnet*          ifp = &(pInfo->arpcom.ac_if);
    struct ether_header*    eh;
    struct mbuf*            mp;
    int                     total_len;
    uint32_t                rxctl;
    uint16_t                rxstat;

    rxstat = (uint16_t)pRD->rdesc0;
    rxctl = pRD->rdesc1;

    /*
     * Bad/error frames will have the RXOK bit cleared.
     * However, there's one error case we want to allow:
     * if a VLAN tagged frame arrives and the chip can't
     * match it against the CAM filter, it considers this
     * a 'VLAN CAM filter miss' and clears the 'RXOK' bit.
     * We don't want to drop the frame though: our VLAN
     * filtering is done in software.
     */
    if (!(rxstat & RSR_RXOK)) {
        if (rxstat & (RSR_RL|RSR_CE|RSR_VIDM))  {
            /*GETB_PRT0(pInfo, "Don't drop CE and RL packet.\n");*/
        }
        else { /*Drop this RD, move to next RD*/
            /*GETB_PRT0(pInfo, "Drop error packet.\n");*/
            return FALSE;
        }
    }

    if (rxstat & (RSR_STP|RSR_EDP)) {
        GETB_PRT0(pInfo, "The received frame span multple RDs\n");
        /*pStats->rx_length_errors++;*/
        return FALSE;
    }



    if (rxstat & RSR_MAR)
        ifp->if_imcasts++;

    if (rxstat & RSR_BAR)
        /*pInfo->adwRMONStats[RMON_BroadcastPkts]++*/ ;

    mp = pRDInfo->mbuf;
    total_len = ((pRD->rdesc0 & 0x3fff0000L) >> 16) - 4;

   /*
    * If allocating a replacement mbuf fails,
    * drop the packet
    */
    if (!getb_alloc_rx_buf(pInfo, idx, NULL)) {
        GETB_PRT0(pInfo, "Can not allocate rx buf\n");
        return FALSE;
    }

    mp->m_pkthdr.rcvif = ifp;
    mp->m_pkthdr.len = mp->m_len = total_len;

    eh = mtod(mp, struct ether_header *);

    /*drop frame not met IEEE 802.3*/
    if (pInfo->hw.flags & GETB_FLAGS_VAL_PKT_LEN) {
        if ((htons(eh->ether_type)>=46) &&
            (htons(eh->ether_type)<=1500)) {
            U16 wSAP=*(((PU16) &eh->ether_type)+1);
            if (wSAP!=0xffff) {
                if (htons(eh->ether_type)!=mp->m_len) {
                    m_freem(mp);
                    return FALSE;
                }
            }
        }
    }

#if __FreeBSD_version >= 400000
    if (pInfo->hw.flags & GETB_FLAGS_RX_CSUM) {
        /* Do Checksum */
        getb_rx_csum(rxctl, mp);
    }
#endif

#if __FreeBSD_version >= 500000
    if (pInfo->hw.flags & GETB_FLAGS_VLANTAG) {
        /* Do VLAN */
        if (rxstat & RSR_VTAG) {
            /* get the tagging info */
            VLAN_INPUT_TAG(ifp, mp, ntohs(rxctl & 0x0000FFFF), /*errcase empty*/);
        }
    }
#endif

    ifp->if_ipackets++;
    ifp->if_ibytes += mp->m_pkthdr.len;


#if __FreeBSD_version < 400000

#if NBPFILTER > 0
        /*
         * Handle BPF listeners. Let the BPF user see the packet, but
         * don't pass it up to the ether_input() layer unless it's
         * a broadcast packet, multicast packet, matches our ethernet
         * address or the interface is in promiscuous mode.
         */
        if (ifp->if_bpf) {
            bpf_mtap(ifp, mp);
            if ((ifp->if_flags & IFF_PROMISC) &&
                (bcmp(eh->ether_dhost, sc->arpcom.ac_enaddr, ETHER_ADDR_LEN) &&
                (eh->ether_dhost[0] & 1) == 0)) {
                m_freem(mp);
                return TRUE;
            }
        }
#endif /*NBPFILTER*/

#elif __FreeBSD_version >=400000 && __FreeBSD_version < 410000
        /*
         * Handle BPF listeners. Let the BPF user see the packet, but
         * don't pass it up to the ether_input() layer unless it's
         * a broadcast packet, multicast packet, matches our ethernet
         * address or the interface is in promiscuous mode.
         */
        if (ifp->if_bpf) {
            bpf_mtap(ifp, mp);
            if ((ifp->if_flags & IFF_PROMISC) &&
                (bcmp(eh->ether_dhost, sc->arpcom.ac_enaddr, ETHER_ADDR_LEN) &&
                (eh->ether_dhost[0] & 1) == 0)) {
                m_freem(mp);
                return TRUE;
            }
        }
#endif /*__FreeBSD_version*/


#if __FreeBSD_version >= 500000
    GET_UNLOCK(pInfo);
    (*ifp->if_input)(ifp, mp);
    GET_LOCK(pInfo);
#else
    /* Remove header from mbuf and pass it on. */
    m_adj(mp, sizeof(struct ether_header));
    ether_input(ifp, eh, mp);
#endif /*__FreeBSD_version >= 500000*/

    return TRUE;
}


static int getb_tx_srv (PGETB_INFO pInfo)
{
    PTX_DESC        pTD;
    int             iQNo;
    int             idx;
    PGETB_TD_INFO   pTDInfo;
    struct  ifnet*  ifp = &(pInfo->arpcom.ac_if);


    for (iQNo=0; iQNo<pInfo->hw.nTxQueues; iQNo++) {

        idx=pInfo->hw.aiTailTDIdx[iQNo];

        while (idx != pInfo->hw.aiCurrTDIdx[iQNo]) {

            pTD = &(pInfo->hw.apTDRings[iQNo][idx]);
            pTDInfo = &(pInfo->apTDInfos[iQNo][idx]);

            if (pTD->tdesc0 & TDESC0_OWN)
                break;

            if (pTD->tdesc0 & TSR_TERR) {
                ifp->if_oerrors++;

#if __FreeBSD_version < 400000
                if (pTD->tdesc0 & TSR_OWC)
                    ifp->if_collisions++;
#endif
            }
            else {
                ifp->if_opackets++;
                ifp->if_obytes += pTDInfo->mbuf->m_pkthdr.len;
            }

#if __FreeBSD_version < 400000
            if (pTD->tdesc0 & TSR_COLS)
                ifp->if_collisions+=(pTD->tdesc0 & 0x000F);
#endif

            if (pTDInfo->mbuf) {
                m_freem(pTDInfo->mbuf);
                pTDInfo->mbuf=NULL;
            }

            pTD->aTdBufs[0].dwBufaddrHi &= ~TDTXBUF_QUE;

            pInfo->hw.iTDUsed[iQNo]--;
            idx = (idx+1) % pInfo->hw.sOpts.nTxDescs;

        } /* while */

        if (idx != pInfo->hw.aiTailTDIdx[iQNo]) {
                pInfo->hw.aiTailTDIdx[iQNo] = idx;
                ifp->if_flags &= ~IFF_OACTIVE;
                ifp->if_timer = 0;
        }

    } /* for */

    return 0;
}


static void getb_error(PGETB_INFO pInfo, u_int32_t status)
{

    if (status & ISR_TXSTLI) {
        printf("txstli %4x.\n", CSR_READ_1(&pInfo->hw, MAC_REG_TXE_SR));
        printf("TD DMA error!\n");
        getb_init(pInfo);
    }

    if (status & ISR_SRCI) {

        pInfo->hw.mii_status=check_connectiontype(&pInfo->hw);
        velocity_print_link_status(&pInfo->hw);

        enable_flow_control_ability(&pInfo->hw);

        /* This is patch for VT3119A1
         * Use TCR_TB2BDIS to prevent 1c_Reset failure in half-duplex mode
         * Only use this in 10HD and 100HD
         */
        if (pInfo->hw.byRevId < REV_ID_VT3216_A0) {
            if (pInfo->hw.mii_status & (VELOCITY_SPEED_100|VELOCITY_SPEED_10)) {
                if (pInfo->hw.mii_status & VELOCITY_DUPLEX_FULL) {
                    BYTE_REG_BITS_OFF(&pInfo->hw, TCR_TB2BDIS, MAC_REG_TCR);
                }
                else {
                    BYTE_REG_BITS_ON(&pInfo->hw, TCR_TB2BDIS, MAC_REG_TCR);
                }
            }
        }

        /* This is patch for VT3119/VT3216
         * Patch HW MIB TXSQEErr Bug, this should only function in 10HD
         */
        if (pInfo->hw.byRevId < REV_ID_VT3284_A0) {
            if ((pInfo->hw.mii_status & VELOCITY_SPEED_10) && !(pInfo->hw.mii_status & VELOCITY_DUPLEX_FULL)) {
                /* Enable count TXSQEErr in 10HD */
                BYTE_REG_BITS_OFF(&pInfo->hw, 0x80, MAC_REG_TESTCFG);
            }
            else {
                /* Disable count TXSQEErr in 10FD, 100HD, 100FD, 1G */
                BYTE_REG_BITS_ON(&pInfo->hw, 0x80, MAC_REG_TESTCFG);
            }
        }

    }

    if (status & ISR_MIBFI)
        velocity_update_hw_mibs(&pInfo->hw);

    if (status & ISR_LSTEI) {
        getb_rx_srv(pInfo);
        mac_rx_queue_run(&pInfo->hw);
        mac_rx_queue_wake(&pInfo->hw);
    }
}


static struct mbuf* defrag_mbuf(struct mbuf* mp)
{
    struct mbuf* m_new;

#if __FreeBSD_version >= 502000
    m_new = m_defrag(mp, M_DONTWAIT);
#else
    MGETHDR(m_new, M_DONTWAIT, MT_DATA);

    if (m_new == NULL)
        return NULL;

    if (mp->m_pkthdr.len > MHLEN) {
        MCLGET(m_new, M_DONTWAIT);
        if (!(m_new->m_flags & M_EXT)) {
            m_freem(m_new);
            return NULL;
        }
    }

    m_copydata(mp, 0, mp->m_pkthdr.len, mtod(m_new, caddr_t));
    m_new->m_pkthdr.len = m_new->m_len = mp->m_pkthdr.len;
    m_freem(mp);
#endif

    return m_new;
}

static void getb_start(struct ifnet* ifp)
{
    PGETB_INFO      pInfo=ifp->if_softc;
    int             iQNo=0;
    struct mbuf*    m_head;
    struct mbuf*    mp;
    PTX_DESC        pTD, pPreTD;
    PGETB_TD_INFO   pTDInfo;
    int             iPreTDIdx, iCurrTDIdx;
    int             iSegments=0;
    struct mbuf     *m_new = NULL;

    GET_LOCK(pInfo);

    iCurrTDIdx=pInfo->hw.aiCurrTDIdx[iQNo];

    while (ifp->if_snd.ifq_head != NULL) {

        IF_DEQUEUE(&ifp->if_snd, m_head);

        if (m_head == NULL)
            break;

        if (AVAIL_TD(&pInfo->hw, iQNo)<=1) {
            getb_tx_srv(pInfo);
        }

        if (AVAIL_TD(&pInfo->hw, iQNo)<=1)  {
            ifp->if_flags |= IFF_OACTIVE;
            IF_PREPEND(&ifp->if_snd, m_head);
            break;
        }

        pTD = &(pInfo->hw.apTDRings[iQNo][iCurrTDIdx]);
        pTDInfo = &(pInfo->apTDInfos[iQNo][iCurrTDIdx]);

        pTD->tdesc1 = cpu_to_le32(TCR_TIC);
        VELOCITY_SET_TD_TCPLS(pTD, TCPLS_NORMAL);
        pTD->aTdBufs[0].dwBufaddrHi &= cpu_to_le32(~TDTXBUF_QUE);

#if __FreeBSD_version >= 400000
        /* Do Checksum */
        if (pInfo->hw.flags & GETB_FLAGS_TX_CSUM) {
            if (m_head->m_pkthdr.csum_flags & CSUM_IP)
                pTD->tdesc1 |= TCR_IPCK;
            if (m_head->m_pkthdr.csum_flags & CSUM_TCP)
                pTD->tdesc1 |= TCR_TCPCK;
            if (m_head->m_pkthdr.csum_flags & CSUM_UDP)
                pTD->tdesc1 |= TCR_UDPCK;
        }
#endif

#if __FreeBSD_version >= 500000
        /* Do VLAN */
        if (pInfo->hw.flags & GETB_FLAGS_VLANTAG) {
            struct m_tag *mtag;
            mtag = VLAN_OUTPUT_TAG(&pInfo->arpcom.ac_if, m_head);
            if (mtag != NULL) {
                pTD->tdesc1 |= (u_int32_t)VLAN_TAG_VALUE(mtag);
                pTD->tdesc1 |= cpu_to_le32(TCR_VETAG);
            }
        }
#endif

        /*
         * Load mbufs to TDs
         */
        if (m_head->m_pkthdr.len < ETH_ZLEN) {
            /*
             * Frame size < GETO_MIN_FRAME_SIZE
             */

            m_new = defrag_mbuf(m_head);
            if (m_new == NULL) {
                IF_PREPEND(&ifp->if_snd, m_head);
                printf("no memory for tx list");
                break;
            }

            m_head = m_new;

            /*
             * we have to make sure to pad short frames out to the minimum frame length
             * ourselves.
             */
            bzero(mtod(m_new, char*)+m_new->m_pkthdr.len, ETH_ZLEN - m_new->m_len);
            m_new->m_pkthdr.len += ETH_ZLEN - m_new->m_len;
            m_new->m_len = m_new->m_pkthdr.len;

            pTD->aTdBufs[iSegments].dwBufAddrLo=cpu_to_le32(vtophys(mtod(m_new, caddr_t)));
            VELOCITY_SET_TD_BUFFER_SIZE(pTD->aTdBufs[iSegments], m_new->m_len);
            VELOCITY_SET_TD_PACKET_SIZE(pTD, m_new->m_len);
            iSegments=1;

        }
        else {
            /*
             * Frame size >= GETO_MIN_FRAME_SIZE
             */
again:
            VELOCITY_SET_TD_PACKET_SIZE(pTD, m_head->m_pkthdr.len);

            for (mp = m_head, iSegments=0; mp != NULL; mp = mp->m_next) {

                if (iSegments>=7)
                    break;

                if (mp->m_len == 0)
                    continue;

                pTD->aTdBufs[iSegments].dwBufAddrLo=cpu_to_le32(vtophys(mtod(mp, vm_offset_t)));
                VELOCITY_SET_TD_BUFFER_SIZE(pTD->aTdBufs[iSegments], mp->m_len);
                iSegments++;
            }

            if (mp!=NULL) {
                /*
                 * We ran out of segments. We have to recopy this
                 * mbuf chain first. Bail out if we can't get the
                 * new buffers.
                 */
                m_new = defrag_mbuf(m_head);
                if (m_new == NULL) {
                    IF_PREPEND(&ifp->if_snd, m_head);
                    printf("no memory for tx list");
                    break;
                }

                m_head = m_new;
                goto again;
            }

        } /*else of if (m_head->m_pkthdr.len < ETH_ZLEN) */


        pTDInfo->mbuf = m_head;
        VELOCITY_SET_TD_CMDZ(pTD, iSegments+1);

        /*
         *Send a copy of the frame to the BPF listener
         */
#if __FreeBSD_version >= 500000

        BPF_MTAP(ifp, m_head);

#elif __FreeBSD_version >= 400000

        if (ifp->if_bpf)
            bpf_mtap(ifp, m_head);

#else

#if NBPFILTER > 0
        if (ifp->if_bpf)
            bpf_mtap(ifp, m_head);
#endif

#endif /*__FreeBSD_version*/


        /*
         * Set Que/Own bits
         */
        iPreTDIdx = iCurrTDIdx-1;
        if (iPreTDIdx<0)
            iPreTDIdx=pInfo->hw.sOpts.nTxDescs-1;

        pPreTD = &(pInfo->hw.apTDRings[iQNo][iPreTDIdx]);

        pTD->tdesc0 |= cpu_to_le32(TDESC0_OWN);
        pPreTD->aTdBufs[0].dwBufaddrHi |= cpu_to_le32(TDTXBUF_QUE);

        pInfo->hw.iTDUsed[iQNo]++;
        iCurrTDIdx = (iCurrTDIdx+1) % pInfo->hw.sOpts.nTxDescs;

    } /* end of while loop */


    if (iCurrTDIdx == pInfo->hw.aiCurrTDIdx[iQNo]) {
        GET_UNLOCK(pInfo);
        return;
    }

    /* Set Tx queue wake */
    mac_tx_queue_wake(&pInfo->hw, iQNo);

    pInfo->hw.aiCurrTDIdx[iQNo] = iCurrTDIdx;
    GET_UNLOCK(pInfo);

    ifp->if_timer = GETB_TX_TIMEOUT;

    return;
}


static int  getb_ioctl(struct ifnet* ifp, u_long cmd, caddr_t data)
{
    int             error = 0;
    struct ifreq*   ifr = (struct ifreq *)data;
    PGETB_INFO      pInfo = ifp->if_softc;

    GET_LOCK(pInfo);

    switch (cmd) {
    case SIOCSIFADDR:
    case SIOCGIFADDR:
        ether_ioctl(ifp, cmd, data);
        break;

    case SIOCSIFMTU:
        if (ifr->ifr_mtu > VELOCITY_MAX_MTU) {
            error = EINVAL;
        } else {
            if (ifr->ifr_mtu + 14 + 4 <=MCLBYTES) {
                ifp->if_mtu = ifr->ifr_mtu;
                pInfo->rx_buf_sz = ifp->if_mtu + 14+4;
                getb_init(pInfo);
            }
            else
                error=EINVAL;
        }
        break;

    case SIOCSIFFLAGS:
        if (ifp->if_flags & IFF_UP) {
            if (ifp->if_flags & IFF_RUNNING && ifp->if_flags & IFF_PROMISC) {
                CSR_WRITE_4(&pInfo->hw, 0xffffffff, MAC_REG_MAR);
                CSR_WRITE_4(&pInfo->hw, 0xffffffff, MAC_REG_MAR+4);
                BYTE_REG_BITS_ON(&pInfo->hw, RCR_AM|RCR_PROM, MAC_REG_RCR);
            } else if (ifp->if_flags & IFF_RUNNING && !(ifp->if_flags & IFF_PROMISC)) {
                BYTE_REG_BITS_OFF(&pInfo->hw, RCR_PROM, MAC_REG_RCR);
            } else
                getb_init(pInfo);
        } else {
            if (ifp->if_flags & IFF_RUNNING) {
                getb_stop(pInfo);
            }
        }
        break;

    case SIOCADDMULTI:
    case SIOCDELMULTI:
        getb_set_multi(pInfo);
        break;

    case SIOCSIFMEDIA:
    case SIOCGIFMEDIA:
        error = ifmedia_ioctl(ifp, ifr, &pInfo->media, cmd);
        break;

#if __FreeBSD_version >= 400000
    case SIOCSIFCAP:
        /* Set Checksum capability by request */
        ifp->if_capenable &= ~(IFCAP_HWCSUM);
        ifp->if_capenable |= ifr->ifr_reqcap & (IFCAP_HWCSUM);

        if (ifp->if_capenable & IFCAP_TXCSUM) {
            ifp->if_hwassist = (CSUM_IP | CSUM_TCP | CSUM_UDP);
            pInfo->hw.flags |= GETB_FLAGS_TX_CSUM;
        }
        else {
            ifp->if_hwassist = 0;
            pInfo->hw.flags &= ~GETB_FLAGS_TX_CSUM;
        }

        if (ifp->if_capenable & IFCAP_RXCSUM) {
            pInfo->hw.flags |= GETB_FLAGS_RX_CSUM;
        }
        else {
            pInfo->hw.flags &= ~GETB_FLAGS_RX_CSUM;
        }

        if (ifp->if_flags & IFF_RUNNING)
            getb_init(pInfo);

        break;
#endif

    default:
        error = EINVAL;

    } /* switch */

    GET_UNLOCK(pInfo);
    return(error);
}


static void getb_watchdog(struct ifnet* ifp)
{
        PGETB_INFO pInfo;

        pInfo = ifp->if_softc;

        GET_LOCK(pInfo);

        printf("getb%d: watchdog timeout\n", pInfo->unit);
        ifp->if_oerrors++;

        getb_tx_srv(pInfo);
        getb_rx_srv(pInfo);

        getb_init(pInfo);

        GET_UNLOCK(pInfo);

        return;
}

#if __FreeBSD_version >= 400000
static void getb_timer(void* arg)
{
    PGETB_INFO  pInfo=arg;
    struct ifnet   *ifp = &pInfo->arpcom.ac_if;

    GET_LOCK(pInfo);

    velocity_update_hw_mibs(&pInfo->hw);

    ifp->if_collisions += pInfo->hw.adwHWMIBCounters[HW_MIB_ifTxEtherCollisions];

    bzero(pInfo->hw.adwHWMIBCounters, sizeof(pInfo->hw.adwHWMIBCounters));
    pInfo->timer_handle = timeout(getb_timer, pInfo, 2*hz);

    GET_UNLOCK(pInfo);
}
#endif

static void getb_init(void *arg)
{
    PGETB_INFO      pInfo=(PGETB_INFO)arg;
    struct ifnet    *ifp;

    GET_LOCK(pInfo);

    getb_stop(pInfo);

    if (!getb_init_rd_ring(pInfo)) {
        GETB_PRT0(pInfo, "getb_init_rd_ring() failed.");
        GET_UNLOCK(pInfo);
        return;
    }

    if (!getb_init_td_ring(pInfo)) {
        GETB_PRT0(pInfo, "getb_init_td_ring() failed.");
        getb_free_rd_ring(pInfo);
        GET_UNLOCK(pInfo);
        return;
    }

    /*
     Initialiation of MAC registers
     */
    getb_init_registers(pInfo, GETB_INIT_COLD);

    ifp = &(pInfo->arpcom.ac_if);

    ifp->if_flags |= IFF_RUNNING;
    ifp->if_flags &= ~IFF_OACTIVE;

#if __FreeBSD_version >= 400000
    pInfo->timer_handle = timeout(getb_timer, pInfo, 2*hz);
#endif

    mac_write_int_mask(pInfo->hw.IntMask, &pInfo->hw);

    mac_enable_int(&pInfo->hw);

    GET_UNLOCK(pInfo);
}


static void getb_stop(void *arg)
{
    PGETB_INFO    pInfo = (PGETB_INFO)arg;
    struct ifnet  *ifp = &(pInfo->arpcom.ac_if);

    GET_LOCK(pInfo);

    ifp->if_flags &= ~(IFF_RUNNING | IFF_OACTIVE);
    ifp->if_timer = 0;

    velocity_shutdown(&pInfo->hw);

#if __FreeBSD_version >= 400000
    untimeout(getb_timer, pInfo, pInfo->timer_handle);
#endif

    getb_free_rd_ring(pInfo);
    getb_free_td_ring(pInfo);

    GET_UNLOCK(pInfo);

    return;
}


static BOOL getb_init_rings(PGETB_INFO pInfo)
{
    int     i;

#if __FreeBSD_version < 400000
    unsigned int        round, pool_sz, rdpool_sz, tdpool_sz;
    caddr_t             roundptr;
    int j;

    rdpool_sz = pInfo->hw.sOpts.nRxDescs * sizeof(RX_DESC);
    tdpool_sz = pInfo->hw.sOpts.nTxDescs * sizeof(TX_DESC);

#endif



#if __FreeBSD_version >= 400000

    /*allocate all RD/TD rings a single pool*/

    pInfo->pool_size=(pInfo->hw.sOpts.nRxDescs * sizeof(RX_DESC) +
                      pInfo->hw.sOpts.nTxDescs * sizeof(TX_DESC)*pInfo->hw.nTxQueues);

    pInfo->pool=contigmalloc(pInfo->pool_size, M_DEVBUF, M_NOWAIT, 0, ~0UL, VELOCITY_DMA_ALIGN, 0);

    if (pInfo->pool==NULL)  {
        GETB_PRT0(pInfo, "Allocate dma memory failed\n");
        return FALSE;
    }

    bzero(pInfo->pool, pInfo->pool_size);

    pInfo->hw.aRDRing=(PRX_DESC)pInfo->pool;

    pInfo->hw.rd_pool_dma=(U32)vtophys((vm_offset_t)pInfo->pool);

    for (i=0; i<pInfo->hw.nTxQueues; i++) {

        pInfo->hw.td_pool_dma[i]=(U32)pInfo->hw.rd_pool_dma+
                              pInfo->hw.sOpts.nRxDescs*sizeof(RX_DESC)+
                              pInfo->hw.sOpts.nTxDescs*sizeof(TX_DESC)*i;

        pInfo->hw.apTDRings[i]=(PTX_DESC)((PU8)pInfo->pool+
                            pInfo->hw.sOpts.nRxDescs*sizeof(RX_DESC)+
                            pInfo->hw.sOpts.nTxDescs*sizeof(TX_DESC)*i);
    }

#elif __FreeBSD_version < 400000

    for (j=0; j<pInfo->hw.nTxQueues+1; j++) {

        if (j==0) pool_sz = rdpool_sz;
        else      pool_sz = tdpool_sz;

        pInfo->rawpool[j] = malloc(pool_sz + VELOCITY_DMA_ALIGN, M_DEVBUF, M_NOWAIT);

        if (pInfo->rawpool[j] == NULL) {
            GETB_PRT0(pInfo, "Allocate dma memory failed\n");
            return FALSE;
        }

        round = (unsigned int)pInfo->rawpool[j] & 0xFF;
        roundptr = pInfo->rawpool[j];

        for (i = 0; i < VELOCITY_DMA_ALIGN; i++) {
            if (round % VELOCITY_DMA_ALIGN) {
                round++;
                roundptr++;
            } else
                break;
        }

        pInfo->pool[j] = roundptr;
        bzero(pInfo->pool[j], pool_sz);
    }

    pInfo->hw.aRDRing=(PRX_DESC)pInfo->pool[0];

    pInfo->hw.rd_pool_dma=(void*)vtophys((vm_offset_t)pInfo->pool[0]);

    for (i=0; i<pInfo->hw.nTxQueues; i++) {
        pInfo->hw.apTDRings[i]=(PTX_DESC)pInfo->pool[i+1];
        pInfo->hw.td_pool_dma[i]=(void*)vtophys((vm_offset_t)pInfo->pool[i+1]);
    }

#endif

    return TRUE;
}


static void getb_free_rings(PGETB_INFO pInfo)
{
#if __FreeBSD_version < 400000
    int i;

    for (i=0; i<pInfo->hw.nTxQueues+1; i++) {
        if (pInfo->pool[i]) {
            free(pInfo->rawpool[i], M_DEVBUF);
            pInfo->pool[i]=NULL;
        }
    }

#else
    if (pInfo->pool) {
        contigfree(pInfo->pool, pInfo->pool_size, M_DEVBUF);
        pInfo->pool=NULL;
    }
#endif

}


static BOOL getb_init_rd_ring(PGETB_INFO pInfo)
{
    int i;
    PRX_DESC        pDesc;
    PGETB_RD_INFO   pRDInfo;

    pInfo->aRDInfo=malloc(sizeof(GETB_RD_INFO)*pInfo->hw.sOpts.nRxDescs, M_DEVBUF, M_NOWAIT);

    if (pInfo->aRDInfo==NULL) {
        GETB_PRT0(pInfo, "Allocate RX desc info failed\n");
        return FALSE;
    }

    bzero(pInfo->aRDInfo, sizeof(GETB_RD_INFO)*pInfo->hw.sOpts.nRxDescs);

    /* Init the RD ring entries */
    for (i = 0; i < pInfo->hw.sOpts.nRxDescs; i++) {
        pDesc = &(pInfo->hw.aRDRing[i]);
        pRDInfo = &(pInfo->aRDInfo[i]);

        if (!getb_alloc_rx_buf(pInfo, i, NULL)) {
            GETB_PRT0(pInfo, "Allocate rx buf failed.\n");
            return FALSE;
        }

        pDesc->rdesc0 |= RDESC0_OWN;
    }

    pInfo->hw.iCurrRDIdx=0;

    return TRUE;
}


static void getb_free_rd_ring(PGETB_INFO pInfo)
{
    int i;

    if (pInfo->aRDInfo==NULL)
        return;

    for (i = 0; i < pInfo->hw.sOpts.nRxDescs; i++) {
        PGETB_RD_INFO   pRDInfo = &(pInfo->aRDInfo[i]);
        if (pRDInfo->mbuf) {
            m_freem(pRDInfo->mbuf);
            pRDInfo->mbuf=NULL;
        }
    }

    if (pInfo->aRDInfo) {
        free(pInfo->aRDInfo, M_DEVBUF);
        pInfo->aRDInfo=NULL;
    }
}


static BOOL getb_init_td_ring(PGETB_INFO pInfo)
{
    int j;

    /* Init the TD ring entries */
    for (j=0; j<pInfo->hw.nTxQueues; j++) {
        pInfo->apTDInfos[j]=malloc(sizeof(GETB_TD_INFO)*pInfo->hw.sOpts.nTxDescs, M_DEVBUF, M_NOWAIT);

        if (pInfo->apTDInfos[j]==NULL) {
            GETB_PRT0(pInfo, "Allocate TX DESC failed.\n");
            return FALSE;
        }

        bzero(pInfo->apTDInfos[j], sizeof(GETB_TD_INFO)*pInfo->hw.sOpts.nTxDescs);
        pInfo->hw.iTDUsed[j]=pInfo->hw.aiTailTDIdx[j]=pInfo->hw.aiCurrTDIdx[j]=0;
    }

    return TRUE;
}


static void getb_free_td_ring(PGETB_INFO pInfo)
{
    int i, j;

    for (j=0; j<pInfo->hw.nTxQueues; j++) {

        if (pInfo->apTDInfos[j]==NULL)
            continue;

        for (i = 0; i < pInfo->hw.sOpts.nTxDescs; i++) {
            PGETB_TD_INFO   pTDInfo = &(pInfo->apTDInfos[j][i]);

            if (pTDInfo==NULL)
                continue;

            if (pTDInfo->mbuf) {
                m_freem(pTDInfo->mbuf);
                pTDInfo->mbuf=NULL;
            }
        }

        free(pInfo->apTDInfos[j], M_DEVBUF);
        pInfo->apTDInfos[j]=NULL;
    }
}

static BOOL getb_alloc_rx_buf(PGETB_INFO pInfo, int idx, struct mbuf* omp)
{
    PRX_DESC        pRD = &(pInfo->hw.aRDRing[idx]);
    PGETB_RD_INFO   pRDInfo = &(pInfo->aRDInfo[idx]);
    struct mbuf*    mp;

    if (omp==NULL) {
#if __FreeBSD_version >= 502000
        mp = m_getcl(M_DONTWAIT, MT_DATA, M_PKTHDR);
        if (mp==NULL) {
            GETB_PRT0(pInfo, "Mbuf allocation failed\n");
            return FALSE;
        }
#else
        MGETHDR(mp, M_DONTWAIT, MT_DATA);

        if (mp==NULL) {
            GETB_PRT0(pInfo, "Mbuf allocation failed\n");
            return FALSE;
        }

        MCLGET(mp, M_DONTWAIT);

        if ((mp->m_flags & M_EXT) == 0) {
             m_freem(mp);
             GETB_PRT0(pInfo, "Mbuf cluster allocation failed\n");
             return FALSE;
        }
#endif
    } else {
        mp = omp;
        mp->m_data = mp->m_ext.ext_buf;
    }

    mp->m_len = mp->m_pkthdr.len = MCLBYTES;

    pRDInfo->mbuf=mp;
    pRDInfo->mbuf_dma=vtophys(mtod(mp, vm_offset_t));

    pRD->rdesc0 = 0;

    VELOCITY_SET_RD_BUFFER_SIZE(pRD, pInfo->rx_buf_sz);
    pRD->rdesc3 |= RDESC3_INTCTL;
    pRD->dwBufAddrLo=cpu_to_le32(pRDInfo->mbuf_dma);
    pRD->rdesc3 &= 0xffff0000L;

    return TRUE;
}

/**********************************************************************/
static void getb_setup_interface(PGETB_INFO pInfo)
{
    struct ifnet   *ifp;

    ifp = &pInfo->arpcom.ac_if;
#if __FreeBSD_version >= 502000
    if_initname(ifp, device_get_name(pInfo->osdep.dev), device_get_unit(pInfo->osdep.dev));
#else
    ifp->if_unit = pInfo->unit;
    ifp->if_name = "getb";
#endif
    ifp->if_mtu = ETHERMTU;
    ifp->if_output = ether_output;
    ifp->if_baudrate = 1000000000;
    ifp->if_init =  getb_init;
    ifp->if_softc = pInfo;
    ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
    ifp->if_ioctl = getb_ioctl;
    ifp->if_start = getb_start;

#if __FreeBSD_version >= 400000
    if (pInfo->hw.flags & (GETB_FLAGS_TX_CSUM|GETB_FLAGS_RX_CSUM)) {
        /* Set Checksum capability */
        ifp->if_hwassist = (CSUM_IP | CSUM_TCP | CSUM_UDP);
        ifp->if_capabilities |= IFCAP_HWCSUM;
    }

#if __FreeBSD_version >= 500000
    if (pInfo->hw.flags & GETB_FLAGS_VLANTAG) {
        /* Set VLAN capability */
        ifp->if_capabilities |= IFCAP_VLAN_MTU|IFCAP_VLAN_HWTAGGING;
    }
#endif

    if (ifp->if_capabilities)
        ifp->if_capenable = ifp->if_capabilities;
#endif

    ifp->if_watchdog = getb_watchdog;
    ifp->if_snd.ifq_maxlen = (pInfo->hw.sOpts.nTxDescs - 1)*7;

#if __FreeBSD_version >= 500000
    /*
     * Call MI attach routine.
     */
    ether_ifattach(ifp, pInfo->arpcom.ac_enaddr);

#elif __FreeBSD_version >= 410000 && __FreeBSD_version < 500000
    /*
     * Call MI attach routine.
     */
    ether_ifattach(ifp, ETHER_BPF_SUPPORTED);

#elif __FreeBSD_version >= 400000 && __FreeBSD_version < 410000
    /*
     * Call MI attach routines.
     */
    if_attach(ifp);
    ether_ifattach(ifp);
    bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));

#else
    /*
     * Call MI attach routines.
     */
    if_attach(ifp);
    ether_ifattach(ifp);

#if NBPFILTER > 0
    bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));
#endif

    at_shutdown(getb_shutdown, pInfo, SHUTDOWN_POST_SYNC);

#endif /*__FreeBSD_version*/


    ifmedia_init(&pInfo->media, IFM_IMASK, getb_media_change, getb_media_status);

    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_10_T, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_10_T | IFM_FDX, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_100_TX, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_100_TX | IFM_FDX, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_1000_TX | IFM_FDX, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_1000_TX, 0, NULL);
    ifmedia_add(&pInfo->media, IFM_ETHER | IFM_AUTO, 0, NULL);
    ifmedia_set(&pInfo->media, IFM_ETHER | IFM_AUTO);

    return;
}


static void getb_media_status(struct ifnet *ifp, struct ifmediareq *ifmr)
{
    PGETB_INFO pInfo= ifp->if_softc;

    pInfo->hw.mii_status=check_connectiontype(&pInfo->hw);

    ifmr->ifm_status = IFM_AVALID;
    ifmr->ifm_active = IFM_ETHER;

    if (pInfo->hw.mii_status & VELOCITY_LINK_FAIL)
        return;

    ifmr->ifm_status |= IFM_ACTIVE;

    if (pInfo->hw.mii_status & VELOCITY_SPEED_1000)
        ifmr->ifm_active |= IFM_1000_TX;
    else if (pInfo->hw.mii_status & VELOCITY_SPEED_100)
        ifmr->ifm_active |= IFM_100_TX;
    else
        ifmr->ifm_active |= IFM_10_T;

    if (pInfo->hw.mii_status & VELOCITY_DUPLEX_FULL)
        ifmr->ifm_active |= IFM_FDX;
    else
        ifmr->ifm_active |= IFM_HDX;
}


static int getb_media_change(struct ifnet *ifp)
{
    PGETB_INFO pInfo = ifp->if_softc;
    struct ifmedia  *ifm = &pInfo->media;
    int spd_dpx=0;

    if (IFM_TYPE(ifm->ifm_media) != IFM_ETHER)
        return(EINVAL);

    switch(IFM_SUBTYPE(ifm->ifm_media)) {
    case IFM_AUTO:
    case IFM_1000_TX:
        spd_dpx=SPD_DPX_AUTO;
        break;

    case IFM_100_TX:
        if ((ifm->ifm_media & IFM_GMASK) == IFM_FDX)
            spd_dpx=SPD_DPX_100_FULL;
        else
            spd_dpx=SPD_DPX_100_HALF;
        break;

    case IFM_10_T:
        if ((ifm->ifm_media & IFM_GMASK) == IFM_FDX)
            spd_dpx=SPD_DPX_10_FULL;
        else
            spd_dpx=SPD_DPX_10_HALF;
        break;

    default:
        GETB_PRT0(pInfo, "Unsupported media type\n");
    }

    pInfo->hw.sOpts.spd_dpx=spd_dpx;
    getb_init(pInfo);

    return(0);
}


/*
  Initialiation of MAC registers
*/
static void getb_init_registers(PGETB_INFO pInfo, GETB_INIT_TYPE InitType)
{
    int         i;

    mac_wol_reset(&pInfo->hw);

    switch (InitType) {
    case GETB_INIT_RESET:
    case GETB_INIT_WOL:
    
        velocity_init_register_reset(&pInfo->hw);

        break;

    case GETB_INIT_COLD:
    default:
        /* do reset */
        velocity_soft_reset(&pInfo->hw);
        mac_eeprom_reload(&pInfo->hw);

        for (i = 0; i < 6; i++)
            CSR_WRITE_1(&pInfo->hw, pInfo->arpcom.ac_enaddr[i], MAC_REG_PAR+i);

        /* EEPROM reloaded will cause bit 0 in MAC_REG_CFGA turned on.
        * it makes MAC receive magic packet automatically. So, driver turn it off.
        */
        BYTE_REG_BITS_OFF(&pInfo->hw, CFGA_PACPI, MAC_REG_CFGA);


        /* set packet filter
           receive directed and broadcast address */
        getb_set_multi(pInfo);

        velocity_init_register_cold(&pInfo->hw);

    } /* switch */
}


static void getb_set_multi(PGETB_INFO pInfo)
{
    struct ifmultiaddr  *ifma;
    struct ifnet        *ifp = &pInfo->arpcom.ac_if;
    int                 mcnt = 0;
    U8                  rx_mode=(RCR_AM|RCR_AB);

    /* First, zot all the multicast entries. */
    bzero(pInfo->hw.abyMCAMMask, sizeof(U8)*8);
    mac_set_cam_mask(&pInfo->hw, pInfo->hw.abyMCAMMask, VELOCITY_MULTICAST_CAM);
    CSR_WRITE_4(&pInfo->hw, 0, MAC_REG_MAR);
    CSR_WRITE_4(&pInfo->hw, 0, MAC_REG_MAR+4);

    if (ifp->if_flags & IFF_ALLMULTI || ifp->if_flags & IFF_PROMISC) {
        velocity_set_all_multi(&pInfo->hw);
        if (ifp->if_flags & IFF_PROMISC)
            rx_mode |= RCR_PROM;
    }
    else {

#if __FreeBSD_version < 500000
        LIST_FOREACH(ifma, &ifp->if_multiaddrs, ifma_link) {
#else
        TAILQ_FOREACH(ifma, &ifp->if_multiaddrs, ifma_link) {
#endif
            if (ifma->ifma_addr->sa_family != AF_LINK)
                continue;

            if (mcnt < pInfo->multicast_limit) {
                mac_set_cam(&pInfo->hw, mcnt,
                    LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
                    VELOCITY_MULTICAST_CAM);

                pInfo->hw.abyMCAMMask[(mcnt)/8] |= 1<<((mcnt) & 7);
            } else {
                velocity_set_all_multi(&pInfo->hw);
                break;
            }
            mcnt++;
        }

        mac_set_cam_mask(&pInfo->hw, pInfo->hw.abyMCAMMask, VELOCITY_MULTICAST_CAM);
    }

    if (ifp->if_mtu > 1500)
        rx_mode |= RCR_AL;

    BYTE_REG_BITS_ON(&pInfo->hw, rx_mode, MAC_REG_RCR);

    return;
}
